@@ -82,7 +82,7 @@ int add_admin(int sock) {
 
 	if ((new = (t_admin*)malloc(sizeof(t_admin))) != NULL) {
 		if ((new->fp = fdopen(sock, "r+")) != NULL) {
-			fprintf(new->fp, "\n\033[02;31mHiawatha v"VERSION" CommandChannel\033[00m\n");
+			fprintf(new->fp, "\n\033[02;31mWelcome to the Hiawatha CommandChannel\033[00m\n");
 			fprintf(new->fp, "Password: \0337\033[00;30;40m"); /* Save cursor position and change color to black */
 			fflush(new->fp);
 
@@ -163,6 +163,7 @@ t_admin *first_admin(void) {
 t_admin *next_admin(void) {
 	if ((loop = next) != NULL) {
 		next = loop->next;
+
 		return loop;
 	}
 
@@ -214,7 +215,8 @@ static void show_help(FILE *fp) {
 				"       clients     : show the connected clients\n"
 				"       status      : show general information\n"
 				"  quit/exit        : quit CommandChannel\n"
-				"  unban <ip>       : unban an IP address\n");
+				"  unban <ip>       : unban an IP address\n"
+				"        all        : unban all IP addresses\n");
 }
 
 static void show_status(FILE *fp) {
@@ -247,7 +249,7 @@ static int run_command(char *line, FILE *fp, t_config *config) {
 
 	if (strcmp(cmd, "ban") == 0) {
 		if (param == NULL) {
-			fprintf(fp, "  ban what?");
+			fprintf(fp, "  ban what?\n");
 		} else {
 			if (split_string(param, &param, &param2, ',') == 0) {
 				time = str2int(param2);
@@ -261,10 +263,10 @@ static int run_command(char *line, FILE *fp, t_config *config) {
 					fprintf(fp, "  error while banning!\n");
 					break;
 				case 0:
-					fprintf(fp, "  IP rebanned\n");
+					fprintf(fp, "  IP rebanned.\n");
 					break;
 				default:
-					fprintf(fp, "  %d IPs banned", count);
+					fprintf(fp, "  %d IPs banned.", count);
 					if (config->kick_on_ban) {
 						fprintf(fp, " and kicked\n");
 					} else {
@@ -292,15 +294,15 @@ static int run_command(char *line, FILE *fp, t_config *config) {
 		if (param == NULL) {
 			fprintf(fp, "  kick what?\n");
 		} else if (strcmp(param, "all") == 0) {
-			fprintf(fp, "   %d clients have been kicked\n", disconnect_clients(config));
+			fprintf(fp, "   %d clients have been kicked.\n", disconnect_clients(config));
 		} else if ((id = str2int(param)) != -1) {
 			if (kick_client(id) == 1) {
-				fprintf(fp, "  client has been kicked\n");
+				fprintf(fp, "  client has been kicked.\n");
 			} else {
 				fprintf(fp, "  client not found!\n");
 			}
 		} else if (parse_ip(param, &ip) != -1) {
-			fprintf(fp, "  %d clients have been kicked\n", kick_ip(&ip));
+			fprintf(fp, "  %d clients have been kicked.\n", kick_ip(&ip));
 		} else if (strcmp(param, "yourself") == 0) {
 			fprintf(fp, "  I can't. I'm a computer.\n");
 		} else if (strcmp(param, "me") == 0) {
@@ -328,11 +330,17 @@ static int run_command(char *line, FILE *fp, t_config *config) {
 		if (param == NULL) {
 			fprintf(fp, "  unban who?\n");
 		} else if (strcmp(param, "all") == 0) {
-			fprintf(fp, "  %d IPs have been unbanned\n", unban_ip(0));
+			default_ipv4(&ip);
+			count = unban_ip(&ip);
+#ifdef HAVE_IPV6
+			default_ipv6(&ip);
+			count += unban_ip(&ip);
+#endif
+			fprintf(fp, "  %d IPs have been unbanned.\n", count);
 		} else if (parse_ip(param, &ip) == -1) {
 			fprintf(fp, "  invalid IP!\n");
 		} else if (unban_ip(&ip) == 1) {
-			fprintf(fp, "  IP has been unbanned\n");
+			fprintf(fp, "  IP has been unbanned.\n");
 		} else {
 			fprintf(fp, "  IP not found!\n");
 		}
