@@ -17,7 +17,7 @@
 #define DELIMITER ';'
 #define TIMESTAMP_SIZE    40
 
-typedef struct type_php_fcgi {
+typedef struct type_php_cgi {
 	char     *executable;
 	char     *chroot;
 	char     *binding;
@@ -26,13 +26,13 @@ typedef struct type_php_fcgi {
 	char     *config_file;
 	t_groups groups;
 
-	struct type_php_fcgi *next;
-} t_php_fcgi;
+	struct type_php_cgi *next;
+} t_php_cgi;
 
 typedef struct {
 	char       *pidfile;
 	char       *forks;
-	t_php_fcgi *php_fcgi;
+	t_php_cgi  *php_cgi;
 	t_keyvalue *envir;
 } t_config;
 
@@ -57,74 +57,81 @@ void log_string(char *mesg) {
 
 /* Start php-cgi as a FastCGI daemon
  */
-int run_php_fcgi(t_php_fcgi *php_fcgi, FILE *pid_fp) {
+int run_php_cgi(t_php_cgi *php_cgi, FILE *pid_fp) {
+	FILE *fp;
 	pid_t pid;
 
-	if ((pid = fork()) == 0) {
-		fclose(pid_fp);
+	if ((fp = fopen(php_cgi->executable, "r")) == NULL) {
+		fprintf(stderr, "PHP binary '%s' not found.\n", php_cgi->executable);
+		log_string("PHP binary not found");
+		return -1;
+	}
+	fclose(fp);
 
-		if (php_fcgi->chroot != NULL) {
-			if (chdir(php_fcgi->chroot) == -1) {
-				exit(EXIT_FAILURE);
-			} else if (chroot(php_fcgi->chroot) == -1) {
-				exit(EXIT_FAILURE);
-			}
-		}
-
-		do {
-			if (setgroups(php_fcgi->groups.number, php_fcgi->groups.array) != -1) {
-				if (setgid(php_fcgi->gid) != -1) {
-					if (setuid(php_fcgi->uid) != -1) {
-						break;
-					}
-				}
-			}
-			fprintf(stderr, "\nError while changing uid/gid!\n");
-			exit(EXIT_FAILURE);
-		} while (false);
-
-		if (close(STDIN_FILENO) == -1) {
-			fprintf(stderr, "Warning: error closing STDIN\n");
-		} else if (open("/dev/null", O_RDONLY) == -1) {
-			fprintf(stderr, "Error redirecting stdin\n");
-			return EXIT_FAILURE;
-		}
-#ifndef DEBUG
-		if (close(STDOUT_FILENO) == -1) {
-			fprintf(stderr, "Warning: error closing STDOUT\n");
-		} else if (open("/dev/null", O_WRONLY) == -1) {
-			fprintf(stderr, "Error redirecting stdout\n");
-			return EXIT_FAILURE;
-		}
-		if (close(STDERR_FILENO) == -1) {
-			fprintf(stderr, "Warning: error closing STDERR\n");
-		} else if (open("/dev/null", O_WRONLY) == -1) {
-			log_string("Error redirecting stderr");
-			return EXIT_FAILURE;
-		}
-#endif
-
-		if (php_fcgi->config_file == NULL) {
-			execlp(php_fcgi->executable, php_fcgi->executable, "-b", php_fcgi->binding, (char*)NULL);
-		} else {
-			execlp(php_fcgi->executable, php_fcgi->executable, "-b", php_fcgi->binding, "-c", php_fcgi->config_file, (char*)NULL);
-		}
-#ifdef DEBUG
-		printf(stderr, "Can't execute %s\n", php_fcgi->executable);
-#else
-		log_string("Can't execute php-cgi");
-#endif
-		exit(EXIT_FAILURE);
+	if ((pid = fork()) != 0) {
+		return pid;
 	}
 
-	return pid;
+	fclose(pid_fp);
+
+	if (php_cgi->chroot != NULL) {
+		if (chdir(php_cgi->chroot) == -1) {
+			exit(EXIT_FAILURE);
+		} else if (chroot(php_cgi->chroot) == -1) {
+			exit(EXIT_FAILURE);
+		}
+	}
+
+	do {
+		if (setgroups(php_cgi->groups.number, php_cgi->groups.array) != -1) {
+			if (setgid(php_cgi->gid) != -1) {
+				if (setuid(php_cgi->uid) != -1) {
+					break;
+				}
+			}
+		}
+		fprintf(stderr, "\nError while changing uid/gid!\n");
+		exit(EXIT_FAILURE);
+	} while (false);
+
+	if (close(STDIN_FILENO) == -1) {
+		fprintf(stderr, "Warning: error closing STDIN\n");
+	} else if (open("/dev/null", O_RDONLY) == -1) {
+		fprintf(stderr, "Error redirecting stdin\n");
+		return EXIT_FAILURE;
+	}
+#ifndef DEBUG
+	if (close(STDOUT_FILENO) == -1) {
+		fprintf(stderr, "Warning: error closing STDOUT\n");
+	} else if (open("/dev/null", O_WRONLY) == -1) {
+		fprintf(stderr, "Error redirecting stdout\n");
+		return EXIT_FAILURE;
+	}
+	if (close(STDERR_FILENO) == -1) {
+		fprintf(stderr, "Warning: error closing STDERR\n");
+	} else if (open("/dev/null", O_WRONLY) == -1) {
+		log_string("Error redirecting stderr");
+		return EXIT_FAILURE;
+	}
+#endif
+
+	if (php_cgi->config_file == NULL) {
+		execlp(php_cgi->executable, php_cgi->executable, "-b", php_cgi->binding, (char*)NULL);
+	} else {
+		execlp(php_cgi->executable, php_cgi->executable, "-b", php_cgi->binding, "-c", php_cgi->config_file, (char*)NULL);
+	}
+
+	fprintf(stderr, "Error while executing %s\n", php_cgi->executable);
+	log_string("Error while executing PHP binary");
+
+	return -1;
 }
 
 /* Parse a configuration line
  */
 int parse_line(t_config *config, char *line) {
 	char *key, *value, *item, *group, *pipe;
-	t_php_fcgi *new;
+	t_php_cgi *new;
 
 	if (split_configline(line, &key, &value) != -1) {
 		strlower(key);
@@ -142,15 +149,15 @@ int parse_line(t_config *config, char *line) {
 				return -1;
 			}
 		} else if (strcmp(key, "setenv") == 0) {
-			if (parse_keyvaluelist(value, &(config->envir), "=") != -1) {
-				return true;
-			}
-		} else if (strcmp(key, "server") == 0) {
-			if ((new = (t_php_fcgi*)malloc(sizeof(t_php_fcgi))) == NULL) {
+			if (parse_keyvaluelist(value, &(config->envir), "=") == -1) {
 				return -1;
 			}
-			new->next = config->php_fcgi;
-			config->php_fcgi = new;
+		} else if (strcmp(key, "server") == 0) {
+			if ((new = (t_php_cgi*)malloc(sizeof(t_php_cgi))) == NULL) {
+				return -1;
+			}
+			new->next = config->php_cgi;
+			config->php_cgi = new;
 
 			/* PHP executable
 			 */
@@ -230,7 +237,7 @@ t_config *read_config(char *config_file) {
 	 */
 	config->pidfile = strdup(PIDFILE_DIR"/php-fcgi.pid");
 	config->forks = strdup("3");
-	config->php_fcgi = NULL;
+	config->php_cgi = NULL;
 	config->envir = NULL;
 
 	if ((fp = fopen(config_file, "r")) == NULL) {
@@ -253,15 +260,38 @@ t_config *read_config(char *config_file) {
 	return config;
 }
 
-void show_help(char *php_fcgi) {
-	printf("Usage: %s [options]\n", php_fcgi);
+/* Verify the configuration
+ */
+bool valid_config(t_config *config) {
+	t_php_cgi *php_cgi;
+	size_t len;
+
+	php_cgi = config->php_cgi;
+	while (php_cgi != NULL) {
+		if ((len = strlen(php_cgi->executable)) >= 8) {
+			if (strcmp(php_cgi->executable + len - 8, "php-fcgi") == 0) {
+				fprintf(stderr, "'%s' is not a PHP binary!\n", php_cgi->executable);
+				return false;
+			}
+		}
+		php_cgi = php_cgi->next;
+	}
+
+	return true;
+}
+
+/* Show help information
+ */
+void show_help(char *php_cgi) {
+	printf("Usage: %s [options]\n", php_cgi);
 	printf("Options: -c <configfile>: the configuration file to be used.\n");
 	printf("         -h: show this information and exit.\n");
 	printf("         -k: kill running FastCGI servers.\n");
 	printf("         -v: show version and exit.\n");
 }
 
-
+/* Main routine
+ */
 int main(int argc, char *argv[]) {
 	t_config *config;
 	t_keyvalue *env;
@@ -299,6 +329,10 @@ int main(int argc, char *argv[]) {
 		return EXIT_FAILURE;
 	}
 
+	if (valid_config(config) == false) {
+		return EXIT_FAILURE;
+	}
+
 	if (kill_servers == false) {
 		/* Start PHP FastCGI servers
 		 */
@@ -311,21 +345,21 @@ int main(int argc, char *argv[]) {
 			env = env->next;
 		}
 
-		if (config->php_fcgi != NULL) {
+		if (config->php_cgi != NULL) {
 			if ((fp = fopen(config->pidfile, "r")) != NULL) {
 				fclose(fp);
-				fprintf(stderr, "A PID-file already exists. Are FastCGI daemons already running?\n");
+				fprintf(stderr, "A PID file exists. Are FastCGI daemons already running?\n");
 				return EXIT_FAILURE;
 			}
 
 			if ((fp = fopen(config->pidfile, "a")) != NULL) {
-				while (config->php_fcgi != NULL) {
-					if ((pid = run_php_fcgi(config->php_fcgi, fp)) != -1) {
+				log_string("Starting PHP FastCGI daemons");
+
+				while (config->php_cgi != NULL) {
+					if ((pid = run_php_cgi(config->php_cgi, fp)) != -1) {
 						fprintf(fp, "%d\n", (int)pid);
-					} else {
-						fprintf(stderr, "Can't start FastCGI server\n");
 					}
-					config->php_fcgi = config->php_fcgi->next;
+					config->php_cgi = config->php_cgi->next;
 				}
 				fclose(fp);
 			} else {
@@ -337,6 +371,8 @@ int main(int argc, char *argv[]) {
 		/* Kill PHP FastCGI servers
 		 */
 		if ((fp = fopen(config->pidfile, "r")) != NULL) {
+			log_string("Stopping PHP FastCGI daemons");
+
 			line[10] = '\0';
 			while (fgets(line, 10, fp) != NULL) {
 				if ((len = strlen(line)) > 0) {
