@@ -668,9 +668,7 @@ int execute_cgi(t_session *session) {
 		return retval;
 	}
 
-	cgi_info.select_timeout.tv_sec = 1;
-	cgi_info.select_timeout.tv_usec = 0;
-	cgi_info.timer = session->host->time_for_cgi;
+	cgi_info.deadline = time(NULL) + session->host->time_for_cgi;
 
 	do {
 #ifdef DEBUG
